@@ -1,6 +1,6 @@
 #
 # Project:   retdec-python
-# Copyright: (c) 2015 by Petr Zemek <s3rvac@gmail.com> and contributors
+# Copyright: (c) 2015-2016 by Petr Zemek <s3rvac@gmail.com> and contributors
 # License:   MIT, see the LICENSE file for more details
 #
 
@@ -19,8 +19,16 @@ class Test(Service):
 
         Does nothing when the authentication succeeds.
         """
+        # Simply send a GET request to /test/echo, and if the authentication
+        # fails, send_get_request() raises AuthenticationError.
+        self.echo()
+
+    def echo(self, **kwargs):
+        """Echoes the given parameters.
+
+        :returns: Echoed `kwargs` (`dict`).
+
+        :raises ``AuthenticationError``: When the authentication fails.
+        """
         conn = self._create_new_api_connection('/test/echo')
-        # We do not need any parameters; simply send a GET request to
-        # /test/echo, and if authentication fails, send_get_request() raises
-        # AuthenticationError.
-        conn.send_get_request()
+        return conn.send_get_request(params=kwargs)
