@@ -1,6 +1,6 @@
 #
 # Project:   retdec-python
-# Copyright: (c) 2015 by Petr Zemek <s3rvac@gmail.com> and contributors
+# Copyright: (c) 2015-2016 by Petr Zemek <s3rvac@gmail.com> and contributors
 # License:   MIT, see the LICENSE file for more details
 #
 # The progress displayers are based on the following script:
@@ -23,7 +23,7 @@
 #     in all copies or substantial portions of the Software.
 #
 
-"""A tool for decompiling files. It uses the library."""
+"""A tool for decompilation of files. It uses the library."""
 
 import abc
 import argparse
@@ -112,28 +112,28 @@ class ProgressLogDisplayer(ProgressDisplayer):
         #    8DRerEdKop
         #    ----------
         #
-        #    Waiting for resources (0%)...                   [OK]
+        #    Waiting for resources (0%)...                 [OK]
         #    Pre-Processing:
-        #        Obtaining file information (5%)...          [OK]
+        #      Obtaining file information (5%)...          [OK]
         #    Front-End:
-        #        Initializing (20%)...                       [OK]
-        #        Creating instruction decoders (22%)...      [OK]
-        #        Detecting statically linked code (25%)...   [OK]
-        #        Instruction decoding (28%)...               [OK]
-        #        Control-flow analysis (31%)...              [OK]
-        #        Data-flow analysis (34%)...                 [OK]
-        #        Type recovery (37%)...                      [OK]
-        #        Generating LLVM IR (40%)...                 [OK]
+        #      Initializing (20%)...                       [OK]
+        #      Creating instruction decoders (22%)...      [OK]
+        #      Detecting statically linked code (25%)...   [OK]
+        #      Instruction decoding (28%)...               [OK]
+        #      Control-flow analysis (31%)...              [OK]
+        #      Data-flow analysis (34%)...                 [OK]
+        #      Type recovery (37%)...                      [OK]
+        #      Generating LLVM IR (40%)...                 [OK]
         #    Middle-End:
-        #        Initializing (45%)...                       [OK]
-        #        Validating the LLVM IR (50%)...             [OK]
-        #        Optimizing the LLVM IR (55%)...             [OK]
+        #      Initializing (45%)...                       [OK]
+        #      Validating the LLVM IR (50%)...             [OK]
+        #      Optimizing the LLVM IR (55%)...             [OK]
         #    Back-End:
-        #        Initializing (60%)...                       [OK]
-        #        Converting the LLVM IR into BIR (65%)...    [OK]
-        #        Optimizing the BIR (70%)...                 [OK]
-        #        Validating the BIR (90%)...                 [OK]
-        #        Generating the target code (95%)...         [OK]
+        #      Initializing (60%)...                       [OK]
+        #      Converting the LLVM IR into BIR (65%)...    [OK]
+        #      Optimizing the BIR (70%)...                 [OK]
+        #      Validating the BIR (90%)...                 [OK]
+        #      Generating the target code (95%)...         [OK]
         #    Done (100%)...
         #
 
@@ -183,12 +183,13 @@ class ProgressLogDisplayer(ProgressDisplayer):
             if phase.part != self._last_part:
                 # Entering a new part.
                 sys.stdout.write('{}:\n'.format(phase.part))
-            phase_str += '    '
+            phase_str += '  '
 
         phase_str += '{} ({}%)...'.format(phase.description, phase.completion)
 
         # Print the phase in an aligned way so the status can be printed
-        # afterwards.
+        # afterwards. The number below has been chosen experimentally based on
+        # the longest phase string that we can have.
         sys.stdout.write('{0:<50} '.format(phase_str))
 
     def _print_decompilation_end(self, d):
@@ -262,16 +263,34 @@ def parse_args(argv):
         formatter_class=argparse.RawDescriptionHelpFormatter
     )
     _add_arguments_shared_by_all_tools(parser)
+    parser.add_argument(
+        '-a', '--architecture',
+        dest='architecture',
+        choices=['x86', 'arm', 'thumb', 'mips', 'pic32', 'powerpc'],
+        help='architecture to force when (de)compiling'
+    )
     parser.add_argument(
         '-b', '--brief',
         dest='brief',
         action='store_true',
         help='print fewer information during the decompilation'
     )
+    parser.add_argument(
+        '-f', '--file-format',
+        dest='file_format',
+        choices=['elf', 'pe'],
+        help='file format to force when compiling'
+    )
+    parser.add_argument(
+        '-l', '--target-language',
+        dest='target_language',
+        choices=['c', 'py'],
+        help='target high-level language'
+    )
     parser.add_argument(
         '-m', '--mode',
         dest='mode',
-        choices={'c', 'bin'},
+        choices=['c', 'bin'],
         help='decompilation mode (default: automatic detection)'
     )
     parser.add_argument(
@@ -280,6 +299,12 @@ def parse_args(argv):
         metavar='DIR',
         help='save the outputs into this directory'
     )
+    parser.add_argument(
+        '-p', '--pdb-file',
+        dest='pdb_file',
+        metavar='FILE',
+        help='PDB file associated with the input file'
+    )
     parser.add_argument(
         '-q', '--quiet',
         dest='quiet',
@@ -328,6 +353,15 @@ def display_download_progress(displayer, file_path):
     displayer.display_download_progress(os.path.basename(file_path))
 
 
+def add_decompilation_param_when_given(args, params, param_name):
+    """Adds a parameter with `param_name` from `args` to `params`, provided
+    that the parameter is set.
+    """
+    param_value = getattr(args, param_name)
+    if param_value is not None:
+        params[param_name] = param_value
+
+
 def main(argv=None):
     """Runs the tool.
 
@@ -342,11 +376,15 @@ def main(argv=None):
         api_key=args.api_key
     )
 
-    decompilation = decompiler.start_decompilation(
-        input_file=args.input_file,
-        mode=args.mode,
-        generate_archive=args.generate_archive
-    )
+    params = {}
+    add_decompilation_param_when_given(args, params, 'input_file')
+    add_decompilation_param_when_given(args, params, 'pdb_file')
+    add_decompilation_param_when_given(args, params, 'mode')
+    add_decompilation_param_when_given(args, params, 'target_language')
+    add_decompilation_param_when_given(args, params, 'architecture')
+    add_decompilation_param_when_given(args, params, 'file_format')
+    add_decompilation_param_when_given(args, params, 'generate_archive')
+    decompilation = decompiler.start_decompilation(**params)
 
     displayer = get_progress_displayer(args)
     displayer.display_decompilation_progress(decompilation)
